<?php

namespace Drupal\commerce_product_review\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Implements the 'commerce_product_review_overall_rating_stars' formatter.
 *
 * This field formatter shows the product's overall rating as stars, based on
 * the rateit.js library.
 *
 * @FieldFormatter(
 *   id = "commerce_product_review_overall_rating_stars",
 *   label = @Translation("Stars"),
 *   field_types = {
 *     "commerce_product_review_overall_rating"
 *   }
 * )
 */
class OverallRatingStarsFormatter extends FormatterBase implements ContainerFactoryPluginInterface, OverallRatingEmptyTextFormatterInterface {

  /**
   * The current active user.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The number formatter.
   *
   * @var \CommerceGuys\Intl\Formatter\NumberFormatterInterface
   */
  protected $numberFormatter;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): static {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->currentUser = $container->get('current_user');
    $instance->numberFormatter = $container->get('commerce_price.number_formatter');
    $instance->entityTypeManager = $container->get('entity_type.manager');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'show_overview_link' => TRUE,
      'show_review_form_link' => TRUE,
      'empty_text' => t('Write the first review'),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $form['show_overview_link'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Link to overview'),
      '#description' => $this->t("If selected, a link to the product's review overview page will be displayed."),
      '#default_value' => $this->getSetting('show_overview_link'),
    ];

    $form['show_review_form_link'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Link to review form'),
      '#description' => $this->t('If selected, a link to the review form will be displayed, so that the visitor can write a product review.'),
      '#default_value' => $this->getSetting('show_review_form_link'),
    ];

    $form['empty_text'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Empty text'),
      '#description' => $this->t('Text displayed, if no published review exists for the given product.'),
      '#default_value' => $this->getSetting('empty_text'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];

    if ($this->getSetting('show_overview_link')) {
      $summary[] = $this->t('Display overview link.');
    }
    else {
      $summary[] = $this->t('Do not display overview link.');
    }

    if ($this->getSetting('show_review_form_link')) {
      $summary[] = $this->t('Display review form link.');
    }
    else {
      $summary[] = $this->t('Do not display review form link.');
    }

    if ($empty_text = $this->getSetting('empty_text')) {
      $summary[] = $this->t('Empty text: @empty_text', ['@empty_text' => $empty_text]);
    }

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];
    /** @var \Drupal\commerce_product\Entity\ProductInterface $product */
    $product = $items->getEntity();
    /** @var \Drupal\commerce_product_review\ProductReviewStorageInterface $product_review_storage */
    $product_review_storage = $this->entityTypeManager->getStorage('commerce_product_review');
    $existing_reviews = $product_review_storage->loadByProductId($product->id(), TRUE);

    $format_options = [
      'maximum_fraction_digits' => $this->getSetting('strip_trailing_zeroes') ? 1 : 3,
    ];

    /** @var \Drupal\commerce_product_review\Plugin\Field\FieldType\OverallRatingItem $item */
    foreach ($items as $delta => $item) {
      $rating = $this->numberFormatter->format($item->score, $format_options);
      $elements[$delta] = [
        '#type' => 'container',
        '#attributes' => ['class' => ['overall-rating']],
        'overall_rating' => [
          '#type' => 'html_tag',
          '#tag' => 'div',
          '#attributes' => [
            'class' => ['rateit'],
            'title' => $this->t('Overall rating: @rating', ['@rating' => $rating]),
            'data-rateit-value' => $item->score,
            // cspell:disable-next-line
            'data-rateit-ispreset' => 'true',
            'data-rateit-readonly' => 'true',
          ],
        ],
      ];

      if ($this->getSetting('show_overview_link')) {
        $elements[$delta]['overview_link'] = [
          '#type' => 'link',
          '#title' => $this->formatPlural(count($existing_reviews), 'View @count review', 'View all @count reviews'),
          '#url' => Url::fromRoute('entity.commerce_product.reviews', ['commerce_product' => $product->id()]),
        ];
      }

      if ($this->getSetting('show_review_form_link')) {
        $existing_own_reviews = $this->currentUser->isAuthenticated() ? $product_review_storage->loadByProductAndUser($product->id(), $this->currentUser->id()) : [];
        if (empty($existing_own_reviews)) {
          $url = Url::fromRoute('entity.commerce_product.review_form', ['commerce_product' => $product->id()]);
          if ($this->currentUser->isAnonymous() && !$this->currentUser->hasPermission('create default commerce_product_review')) {
            $url = Url::fromRoute('user.login', [], ['query' => ['destination' => $url->toString()]]);
          }
          $elements[$delta]['review_form_link'] = [
            '#type' => 'link',
            '#title' => $this->t('Write your own review'),
            '#url' => $url,
          ];
        }
      }
    }
    $elements['#attached']['library'] = ['commerce_product_review/rateit-js'];

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function getEmptyText() {
    return $this->getSetting('empty_text');
  }

}
